"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Entity = void 0;
const BinaryWriter_js_1 = require("./BinaryWriter.js");
const Prefab_js_1 = require("./Prefab.js");
const PhysicalMaterialPartComponent_js_1 = require("./components/PhysicalMaterialPartComponent.js");
const ComponentHash_js_1 = require("./types/ComponentHash.js");
const PhysicalMaterialPartHash_js_1 = require("./types/PhysicalMaterialPartHash.js");
const readComponents_js_1 = require("./utils/readComponents.js");
const writeComponents_js_1 = require("./utils/writeComponents.js");
/**
 * An "item" inside a prefab composition. The `Entity` can have its own components to alter its
 * behaviour. A child prefab of the composition may attach to an `Entity`, forming an hierarchical
 * structure inside the composition. This is primarily used in _A Township Tale_'s crafting system.
 *
 * @see [Class: `Entity`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/Entity.md)
 * @since v3.0.0
 *
 * @example
 * import { Entity } from 'att-string-transcoder';
 *
 * const torchFireEntity = new Entity<'Torch'>('Fire_30100');
 * // or
 * const torchFireEntity = Entity.fromBinary<'Torch'>(myBinaryString);
 */
class Entity {
    /**
     * The hash of the entity.
     *
     * @since v3.0.0
     *
     * @example
     * import { Entity } from 'att-string-transcoder';
     *
     * const entity = new Entity<'Torch'>('Fire_30100');
     *
     * const hash = entity.hash;
     * // `hash` is `30100`
     */
    hash;
    /**
     * The name of the entity.
     *
     * @since v3.0.0
     *
     * @example
     * import { Entity } from 'att-string-transcoder';
     *
     * const entity = new Entity<'Torch'>('Fire_30100');
     *
     * const name = entity.name;
     * // `name` is `'Fire'`
     */
    name;
    /**
     * The alive state of the entity.
     *
     * @since v3.0.0
     *
     * @example
     * import { Entity } from 'att-string-transcoder';
     *
     * const entity = new Entity<'Torch'>('Fire_30100');
     *
     * const isAlive = entity.isAlive;
     * // `isAlive` is `true`
     */
    isAlive;
    /**
     * A map of the stored components.
     *
     * Provides access to the components stored in this entity. Components are keyed to their
     * respective names, unless its data contained an unrecognised hash. In that case, the component
     * will be stored in an array under the `Unknown` key.
     *
     * @since v3.0.0
     *
     * @example
     * import { Entity } from 'att-string-transcoder';
     *
     * const entity = new Entity<'Torch'>('Fire_30100');
     *
     * const components = entity.components;
     * // `components` is `{ Unknown: [] }`
     */
    components;
    /**
     * Creates a new `Entity` object configured with the passed in configuration.
     *
     * @see [Class: `Entity`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/Entity.md)
     * @since v3.0.0
     *
     * @example
     * import { Entity } from 'att-string-transcoder';
     *
     * const torchFireEntity = new Entity<'Torch'>('Fire_30100', {
     *   hash: 30100,
     *   isAlive: true,
     *   components: {}
     * });
     */
    constructor(key, { hash, isAlive, components } = {}) {
        const resolvedName = key === 'Unknown' ? String(key) : String(key).split('_').slice(0, -1).join('_');
        const resolvedHash = resolvedName === 'Unknown' ? hash : Number(String(key).split('_').slice(-1));
        if (typeof resolvedHash === 'undefined') {
            throw new Error('You must manually pass an entity hash when instantiating an "Unknown" Entity.');
        }
        this.hash = resolvedHash;
        this.name = resolvedName;
        this.isAlive = isAlive ?? true;
        this.components = {
            ...components,
            Unknown: components?.Unknown ?? []
        };
    }
    /**
     * Adds a `Component` to the entity. Will override any existing component with that name.
     *
     * @since v3.1.0
     *
     * @example
     * import { Entity, PhysicalMaterialPartComponent, PhysicalMaterialPartHash } from 'att-string-transcoder';
     *
     * const entity = new Entity<'Standard_Side_Pouch_Attachment'>('standard_sidePouch_backPin_L1_7968');
     * const component = new PhysicalMaterialPartComponent({ version: 1, materialHash: PhysicalMaterialPartHash.Iron });
     *
     * entity.addComponent(component);
     */
    addComponent(component) {
        this.components = {
            ...this.components,
            [component.name]: component
        };
        return this;
    }
    /**
     * Reads the binary string data and returns an instantiated entity.
     *
     * @since v3.0.0
     *
     * @example
     * import { Entity } from 'att-string-transcoder';
     *
     * const torchFireEntity = Entity.fromBinary<'Torch'>(binaryReader, { key, hash });
     */
    static fromBinary(reader, { key, hash, componentVersions }) {
        const isAlive = reader.readBoolean();
        const components = (0, readComponents_js_1.readComponents)(reader, componentVersions);
        return new Entity(key, {
            hash,
            isAlive,
            components
        });
    }
    /**
     * Gets the entity's physical material.
     *
     * @since v3.1.0
     *
     * @example
     * import { Entity, PhysicalMaterialPartHash } from 'att-string-transcoder';
     *
     * const entity = new Entity<'Standard_Side_Pouch_Attachment'>('standard_sidePouch_backPin_L1_7968');
     *
     * const materialHash = entity.getMaterial();
     * const materialName = PhysicalMaterialPartHash[materialHash];
     */
    getMaterial() {
        return this.components.PhysicalMaterialPart?.materialHash ?? 0;
    }
    /**
     * Removes all components on this entity.
     *
     * @since v3.1.0
     *
     * @example
     * import { Entity } from 'att-string-transcoder';
     *
     * const entity = new Entity<'Standard_Side_Pouch_Attachment'>('standard_sidePouch_backPin_L1_7968');
     *
     * entity.removeAllComponents();
     */
    removeAllComponents() {
        this.components = { Unknown: [] };
        return this;
    }
    removeComponent(componentArg) {
        if (typeof componentArg === 'undefined') {
            throw new Error('You must pass a component hash or name to remove from this entity.');
        }
        const componentName = typeof componentArg === 'string'
            ? componentArg
            : ComponentHash_js_1.ComponentHash[componentArg];
        if (typeof componentName === 'undefined') {
            for (const component of Object.values(this.components)) {
                if (Array.isArray(component)) {
                    this.components.Unknown = component.filter(unknownComponent => unknownComponent.hash !== componentArg);
                }
                else if (component.hash === componentArg) {
                    delete this.components[component.name];
                }
            }
        }
        else {
            if (componentName in this.components) {
                delete this.components[componentName];
            }
            else {
                const unknownComponentIndex = this.components.Unknown.findIndex(component => component.name === componentArg);
                if (unknownComponentIndex > -1) {
                    this.components.Unknown = this.components.Unknown.toSpliced(unknownComponentIndex, 1);
                }
            }
        }
        return this;
    }
    setMaterial(materialArg) {
        if (typeof materialArg === 'undefined') {
            throw new Error('You must pass a PhysicalMaterialPartHash to set as the material.');
        }
        const version = this.components.PhysicalMaterialPart?.version ?? Prefab_js_1.FALLBACK_PHYSICAL_MATERIAL_PART_VERSION;
        const materialHash = typeof materialArg === 'number' ? materialArg : PhysicalMaterialPartHash_js_1.PhysicalMaterialPartHash[materialArg];
        this.components.PhysicalMaterialPart = new PhysicalMaterialPartComponent_js_1.PhysicalMaterialPartComponent({
            ...this.components.PhysicalMaterialPart,
            version,
            materialHash
        });
        return this;
    }
    /**
     * Returns a `BinaryString` representation of the entity.
     *
     * @since v3.0.0
     *
     * @example
     * import { ComponentHash, Entity } from 'att-string-transcoder';
     *
     * const torchFireEntity = new Entity<'Torch'>('Fire_30100');
     *
     * const componentVersions = new Map<number, number>([
     *   [ComponentHash.NetworkRigidbody, 1],
     *   [ComponentHash.PhysicalMaterialPart, 1],
     *   [ComponentHash.Pickup, 2]
     *   // etc...
     * ]);
     *
     * const binaryString = torchFireEntity.toBinary(componentVersions);
     */
    toBinary(componentVersions) {
        const writer = new BinaryWriter_js_1.BinaryWriter();
        /**
         * @property {boolean} isAlive
         */
        writer.writeBoolean(this.isAlive);
        /**
         * @property {PrefabComponents} components
         */
        (0, writeComponents_js_1.writeComponents)(writer, this.components, componentVersions);
        return writer.flush();
    }
    /**
     * Writes a `BinaryString` representation of the entity to the given `BinaryWriter`, including the
     * entity hash and data length.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryWriter, ComponentHash, Entity } from 'att-string-transcoder';
     *
     * const torchFireEntity = new Entity<'Torch'>('Fire_30100');
     *
     * const componentVersions = new Map<number, number>([
     *   [ComponentHash.NetworkRigidbody, 1],
     *   [ComponentHash.PhysicalMaterialPart, 1],
     *   [ComponentHash.Pickup, 2]
     *   // etc...
     * ]);
     *
     * const writer = new BinaryWriter();
     *
     * torchFireEntity.write(writer, componentVersions);
     */
    write(writer, componentVersions) {
        const data = this.toBinary(componentVersions);
        /**
         * @property {number} hash
         */
        writer.writeUnsignedInteger(this.hash);
        /**
         * @property {number} dataLength
         */
        writer.writeUnsignedInteger(data.length);
        /**
         * @property {BinaryString} data
         */
        writer.writeBits(data);
    }
}
exports.Entity = Entity;
