"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Component = void 0;
/**
 * Base class for all components. This class should not be used directly. Use any of its derived
 * classes instead.
 *
 * @see [Class: `Component`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/Component.md)
 * @see [API Reference Documentation](https://github.com/mdingena/att-string-transcoder/tree/main/docs/README.md)
 */
class Component {
    /**
     * The hash of the component.
     *
     * @example
     * import { DerivedComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new DerivedComponent({ version: componentVersion });
     *
     * const hash = component.hash;
     */
    hash;
    /**
     * The name of the component.
     *
     * @example
     * import { DerivedComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new DerivedComponent({ version: componentVersion });
     *
     * const name = component.name;
     */
    name;
    /**
     * The version of the component. This controls the shape of the component data and its
     * (de)serialisation sequence when reading or writing binary data.
     *
     * @example
     * import { DerivedComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new DerivedComponent({ version: componentVersion });
     *
     * const version = component.version;
     * // `version` is `1`
     */
    version;
    /**
     * Creates a versioned component. This constructor is used internally. You shouldn't create any
     * instances of this base class.
     *
     * @see [Class: `Component`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/Component.md)
     * @see [API Reference Documentation](https://github.com/mdingena/att-string-transcoder/tree/main/docs/README.md)
     */
    constructor({ hash, name, version }) {
        this.hash = hash;
        this.name = name;
        this.version = version;
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    static fromBinary(_reader, _version) {
        throw new Error('Cannot instantiate base Component class from binary string data. You must call the fromBinary() method on the derived Component class.');
    }
    /**
     * Returns a `BinaryString` representation of the component.
     *
     * @example
     * import { DerivedComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new DerivedComponent({ version: componentVersion });
     *
     * const binaryString = component.toBinary(componentVersion);
     */
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    toBinary(_version = this.version) {
        throw new Error('Cannot encode base Component class. You must call the toBinary() method on the derived Component class.');
    }
    /**
     * Writes a `BinaryString` representation of the component to the given `BinaryWriter`, including
     * the component hash and data length.
     *
     * @example
     * import { BinaryWriter, DerivedComponent } from 'att-string-transcoder';
     *
     * const writer = new BinaryWriter();
     * const componentVersion = 1;
     *
     * const component = new DerivedComponent({ version: componentVersion });
     *
     * component.write(writer, componentVersion);
     */
    write(writer, version = this.version) {
        const data = this.toBinary(version);
        /**
         * @property {number} hash
         */
        writer.writeUnsignedInteger(this.hash);
        /**
         * @property {number} dataLength
         */
        writer.writeUnsignedInteger(data.length);
        /**
         * @property {BinaryString} data
         */
        writer.writeBits(data);
    }
}
exports.Component = Component;
