"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FuseComponent = void 0;
const BinaryWriter_js_1 = require("../BinaryWriter.js");
const ComponentHash_js_1 = require("../types/ComponentHash.js");
const Component_js_1 = require("./Component.js");
/**
 * @see [Class: `FuseComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/FuseComponent.md)
 * @since v3.0.0
 * @version 1
 *
 * @example
 * import { FuseComponent } from 'att-string-transcoder';
 *
 * const componentVersion = 1;
 * const component = new FuseComponent({ version: componentVersion });
 */
class FuseComponent extends Component_js_1.Component {
    /**
     * Whether the fuse is finished or not.
     *
     * @since v1
     *
     * @example
     * import { FuseComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new FuseComponent({ version: componentVersion });
     *
     * const isFinished = component.isFinished;
     * // `isFinished` is `false`
     */
    isFinished;
    /**
     * Whether the fuse is lit or not.
     *
     * @since v1
     *
     * @example
     * import { FuseComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new FuseComponent({ version: componentVersion });
     *
     * const isLit = component.isLit;
     * // `isLit` is `false`
     */
    isLit;
    /**
     * How much of the fuse is left.
     *
     * @since v1
     *
     * @example
     * import { FuseComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new FuseComponent({ version: componentVersion });
     *
     * const currentFuseAmount = component.currentFuseAmount;
     * // `currentFuseAmount` is `1`
     */
    currentFuseAmount;
    /**
     * @see [Class: `FuseComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/FuseComponent.md)
     * @since v3.0.0
     * @version 1
     *
     * @example
     * import { FuseComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new FuseComponent({ version: componentVersion });
     */
    constructor({ version, isFinished, isLit, currentFuseAmount }) {
        const hash = ComponentHash_js_1.ComponentHash.Fuse;
        const name = 'Fuse';
        super({ hash, name, version });
        this.isFinished = isFinished ?? false;
        this.isLit = isLit ?? false;
        this.currentFuseAmount = currentFuseAmount ?? 1;
    }
    static fromBinary(reader, version) {
        const props = { version };
        /**
         * @property {boolean} isFinished
         * @since v1
         */
        if (props.version >= 1) {
            props.isFinished = reader.readBoolean();
        }
        /**
         * @property {boolean} isLit
         * @since v1
         */
        if (props.version >= 1) {
            props.isLit = reader.readBoolean();
        }
        /**
         * @property {float} currentFuseAmount
         * @since v1
         */
        if (props.version >= 1) {
            props.currentFuseAmount = reader.readFloat();
        }
        return new FuseComponent(props);
    }
    toBinary(version = this.version) {
        const writer = new BinaryWriter_js_1.BinaryWriter();
        /**
         * @property {boolean} isFinished
         * @since v1
         */
        if (version >= 1) {
            writer.writeBoolean(this.isFinished);
        }
        /**
         * @property {boolean} isLit
         * @since v1
         */
        if (version >= 1) {
            writer.writeBoolean(this.isLit);
        }
        /**
         * @property {float} currentFuseAmount
         * @since v1
         */
        if (version >= 1) {
            writer.writeFloat(this.currentFuseAmount);
        }
        return writer.flush();
    }
}
exports.FuseComponent = FuseComponent;
