"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HeatSourceBaseComponent = void 0;
const BinaryWriter_js_1 = require("../BinaryWriter.js");
const ComponentHash_js_1 = require("../types/ComponentHash.js");
const Component_js_1 = require("./Component.js");
const HUNDRED_YEARS_TICKS = 31557600000000000;
/**
 * @see [Class: `HeatSourceBaseComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/HeatSourceBaseComponent.md)
 * @since v3.0.0
 * @version 1
 *
 * @example
 * import { HeatSourceBaseComponent } from 'att-string-transcoder';
 *
 * const componentVersion = 1;
 * const component = new HeatSourceBaseComponent({ version: componentVersion });
 */
class HeatSourceBaseComponent extends Component_js_1.Component {
    /**
     * Whether this heat source is lit and active or not.
     *
     * @since v1
     *
     * @example
     * import { HeatSourceBaseComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new HeatSourceBaseComponent({ version: componentVersion });
     *
     * const isLit = component.isLit;
     */
    isLit;
    /**
     * Progress of the heat source until extinguishing.
     *
     * @since v1
     *
     * @example
     * import { HeatSourceBaseComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new HeatSourceBaseComponent({ version: componentVersion });
     *
     * const progress = component.progress;
     * // `progress` is `0`
     */
    progress;
    /**
     * @since v1
     *
     * @example
     * import { HeatSourceBaseComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new HeatSourceBaseComponent({ version: componentVersion });
     *
     * const time = component.time;
     * // `time` is `31557600000000000`
     */
    time;
    /**
     * @see [Class: `HeatSourceBaseComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/HeatSourceBaseComponent.md)
     * @since v3.0.0
     * @version 1
     *
     * @example
     * import { HeatSourceBaseComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new HeatSourceBaseComponent({ version: componentVersion });
     */
    constructor({ version, isLit, progress, time }) {
        const hash = ComponentHash_js_1.ComponentHash.HeatSourceBase;
        const name = 'HeatSourceBase';
        super({ hash, name, version });
        this.isLit = isLit ?? true;
        this.progress = progress ?? 0;
        this.time = time ?? HUNDRED_YEARS_TICKS;
    }
    static fromBinary(reader, version) {
        const props = { version };
        /**
         * @property {boolean} isLit
         * @since v1
         */
        if (props.version >= 1) {
            props.isLit = reader.readBoolean();
        }
        /**
         * @property {float} progress
         * @since v1
         */
        if (props.version >= 1) {
            props.progress = reader.readFloat();
        }
        /**
         * @property {number} time
         * @since v1
         */
        if (props.version >= 1) {
            props.time = reader.readUnsignedLong();
        }
        return new HeatSourceBaseComponent(props);
    }
    toBinary(version = this.version) {
        const writer = new BinaryWriter_js_1.BinaryWriter();
        /**
         * @property {boolean} isLit
         * @since v1
         */
        if (version >= 1) {
            writer.writeBoolean(this.isLit);
        }
        /**
         * @property {float} progress
         * @since v1
         */
        if (version >= 1) {
            writer.writeFloat(this.progress);
        }
        /**
         * @property {number} time
         * @since v1
         */
        if (version >= 1) {
            writer.writeUnsignedLong(this.time);
        }
        return writer.flush();
    }
}
exports.HeatSourceBaseComponent = HeatSourceBaseComponent;
