"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LogicGateReceiverComponent = void 0;
const BinaryWriter_js_1 = require("../BinaryWriter.js");
const ComponentHash_js_1 = require("../types/ComponentHash.js");
const LogicOperator_js_1 = require("../types/LogicOperator.js");
const Component_js_1 = require("./Component.js");
/**
 * @see [Class: `LogicGateReceiverComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/LogicGateReceiverComponent.md)
 * @since v3.0.0
 * @version 2
 *
 * @example
 * import { LogicGateReceiverComponent } from 'att-string-transcoder';
 *
 * const componentVersion = 2;
 * const component = new LogicGateReceiverComponent({ version: componentVersion });
 */
class LogicGateReceiverComponent extends Component_js_1.Component {
    /**
     * The identifiers of the `Logic*Sender` components that are connected to this `LogicGateReceiver`.
     *
     * @since v2
     *
     * @example
     * import { LogicGateReceiverComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new LogicGateReceiverComponent({ version: componentVersion });
     *
     * const senders = component.senders;
     * // `senders` is `[]`
     */
    senders;
    /**
     * The type of operation to perform on the received inputs.
     *
     * @since v2
     *
     * @example
     * import { LogicGateReceiverComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new LogicGateReceiverComponent({ version: componentVersion });
     *
     * const operationType = component.operationType;
     * // `operationType` is `0` (AND)
     */
    operationType;
    /**
     * Whether to inverse the output of this gate.
     *
     * @since v2
     *
     * @example
     * import { LogicGateReceiverComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new LogicGateReceiverComponent({ version: componentVersion });
     *
     * const isInversedOutputSaved = component.isInversedOutputSaved;
     * // `isInversedOutputSaved` is `false`
     */
    isInversedOutputSaved;
    /**
     * @see [Class: `LogicGateReceiverComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/LogicGateReceiverComponent.md)
     * @since v3.0.0
     * @version 2
     *
     * @example
     * import { LogicGateReceiverComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new LogicGateReceiverComponent({ version: componentVersion });
     */
    constructor({ version, senders, operationType, isInversedOutputSaved }) {
        const hash = ComponentHash_js_1.ComponentHash.LogicGateReceiver;
        const name = 'LogicGateReceiver';
        super({ hash, name, version });
        this.senders = senders ?? [];
        this.operationType = operationType ?? LogicOperator_js_1.LogicOperator.And;
        this.isInversedOutputSaved = isInversedOutputSaved ?? false;
    }
    static fromBinary(reader, version) {
        const props = { version };
        /**
         * @property {number[]} senders
         * @since v2
         */
        if (props.version >= 2) {
            const sendersLength = reader.readUnsignedInteger();
            props.senders = [];
            for (let sendersIndex = 0; sendersIndex < sendersLength; ++sendersIndex) {
                /**
                 * @property {number} sender
                 * @since v2
                 */
                props.senders.push(reader.readUnsignedInteger());
            }
        }
        /**
         * @property {number} operationType
         * @since v2
         */
        if (props.version >= 2) {
            props.operationType = reader.readSignedInteger();
        }
        /**
         * @property {number} isInversedOutputSaved
         * @since v2
         */
        if (props.version >= 2) {
            props.isInversedOutputSaved = reader.readBoolean();
        }
        return new LogicGateReceiverComponent(props);
    }
    toBinary(version = this.version) {
        const writer = new BinaryWriter_js_1.BinaryWriter();
        /**
         * @property {number[]} senders
         * @since v2
         */
        if (version >= 2) {
            writer.writeUnsignedInteger(this.senders.length);
            for (const sender of this.senders) {
                /**
                 * @property {number} sender
                 * @since v2
                 */
                writer.writeUnsignedInteger(sender);
            }
        }
        /**
         * @property {number} operationType
         * @since v2
         */
        if (version >= 2) {
            writer.writeSignedInteger(this.operationType);
        }
        /**
         * @property {number} isInversedOutputSaved
         * @since v2
         */
        if (version >= 2) {
            writer.writeBoolean(this.isInversedOutputSaved);
        }
        return writer.flush();
    }
}
exports.LogicGateReceiverComponent = LogicGateReceiverComponent;
