"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LogicVector3SenderComponent = void 0;
const BinaryWriter_js_1 = require("../BinaryWriter.js");
const ComponentHash_js_1 = require("../types/ComponentHash.js");
const Component_js_1 = require("./Component.js");
/**
 * @see [Class: `LogicVector3SenderComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/LogicVector3SenderComponent.md)
 * @since v3.0.0
 * @version 1
 *
 * @example
 * import { LogicVector3SenderComponent } from 'att-string-transcoder';
 *
 * const componentVersion = 1;
 * const component = new LogicVector3SenderComponent({ version: componentVersion });
 */
class LogicVector3SenderComponent extends Component_js_1.Component {
    /**
     * We're not quite sure what this property does.
     *
     * @since v1
     *
     * @example
     * import { LogicVector3SenderComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new LogicVector3SenderComponent({ version: componentVersion });
     *
     * const updatedValue = component.updatedValue;
     * // `updatedValue` is `3171294583`
     */
    updatedValue;
    /**
     * We're not quite sure what this property does.
     *
     * @since v1
     *
     * @example
     * import { LogicVector3SenderComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new LogicVector3SenderComponent({ version: componentVersion });
     *
     * const changedExternally = component.changedExternally;
     * // `changedExternally` is `32`
     */
    changedExternally;
    /**
     * The identifier of this `LogicVector3Sender` that a `LogicVector3Receiver` connects to.
     *
     * @since v1
     *
     * @example
     * import { LogicVector3SenderComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new LogicVector3SenderComponent({ version: componentVersion });
     *
     * const identifier = component.identifier;
     * // `identifier` is `0`
     */
    identifier;
    /**
     * The output value of this `LogicVector3Sender` that a `LogicVector3Receiver` reads as its input.
     *
     * @since v1
     *
     * @example
     * import { LogicVector3SenderComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new LogicVector3SenderComponent({ version: componentVersion });
     *
     * const value = component.value;
     * // `value` is `{ x: 0, y: 0, z: 0 }`
     */
    value;
    /**
     * @see [Class: `LogicVector3SenderComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/LogicVector3SenderComponent.md)
     * @since v3.0.0
     * @version 1
     *
     * @example
     * import { LogicVector3SenderComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new LogicVector3SenderComponent({ version: componentVersion });
     */
    constructor({ version, updatedValue, changedExternally, identifier, value }) {
        const hash = ComponentHash_js_1.ComponentHash.LogicVector3Sender;
        const name = 'LogicVector3Sender';
        super({ hash, name, version });
        this.updatedValue = updatedValue ?? 3171294583;
        this.changedExternally = changedExternally ?? 32;
        this.identifier = identifier ?? 0;
        this.value = value ?? { x: 0, y: 0, z: 0 };
    }
    static fromBinary(reader, version) {
        const props = { version };
        /**
         * @property {number} updatedValue
         * @since v1
         */
        if (props.version >= 1) {
            props.updatedValue = reader.readUnsignedInteger();
        }
        /**
         * @property {number} changedExternally
         * @since v1
         */
        if (props.version >= 1) {
            props.changedExternally = reader.readUnsignedInteger();
        }
        /**
         * @property {number} identifier
         * @since v1
         */
        if (props.version >= 1) {
            props.identifier = reader.readUnsignedInteger();
        }
        /**
         * @property {Vector3} value
         * @since v1
         */
        if (props.version >= 1) {
            props.value = {
                x: reader.readFloat(),
                y: reader.readFloat(),
                z: reader.readFloat()
            };
        }
        return new LogicVector3SenderComponent(props);
    }
    toBinary(version = this.version) {
        const writer = new BinaryWriter_js_1.BinaryWriter();
        /**
         * @property {number} updatedValue
         * @since v1
         */
        if (version >= 1) {
            writer.writeUnsignedInteger(this.updatedValue);
        }
        /**
         * @property {number} changedExternally
         * @since v1
         */
        if (version >= 1) {
            writer.writeUnsignedInteger(this.changedExternally);
        }
        /**
         * @property {number} identifier
         * @since v1
         */
        if (version >= 1) {
            writer.writeUnsignedInteger(this.identifier);
        }
        /**
         * @property {Vector3} value
         * @since v1
         */
        if (version >= 1) {
            writer.writeFloat(this.value.x);
            writer.writeFloat(this.value.y);
            writer.writeFloat(this.value.z);
        }
        return writer.flush();
    }
}
exports.LogicVector3SenderComponent = LogicVector3SenderComponent;
