"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NetworkRigidbodyComponent = void 0;
const BinaryWriter_js_1 = require("../BinaryWriter.js");
const ComponentHash_js_1 = require("../types/ComponentHash.js");
const Component_js_1 = require("./Component.js");
/**
 * Controls physical behaviour of the `Prefab` it is attached to.
 *
 * @see [Class: `NetworkRigidbodyComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/NetworkRigidbodyComponent.md)
 * @since v3.0.0
 * @version 1
 *
 * @example
 * import { NetworkRigidbodyComponent } from 'att-string-transcoder';
 *
 * const componentVersion = 1;
 * const component = new NetworkRigidbodyComponent({ version: componentVersion });
 */
class NetworkRigidbodyComponent extends Component_js_1.Component {
    /**
     * Position of the prefab this component is attached to.
     *
     * @since v1
     *
     * @example
     * import { NetworkRigidbodyComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new NetworkRigidbodyComponent({ version: componentVersion });
     *
     * const position = component.position;
     * // `position` is `{ x: 0, y: 0, z: 0 }`
     */
    position;
    /**
     * Rotation of the prefab this component is attached to.
     *
     * @since v1
     *
     * @example
     * import { NetworkRigidbodyComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new NetworkRigidbodyComponent({ version: componentVersion });
     *
     * const rotation = component.rotation;
     * // `rotation` is `{ x: 0, y: 0, z: 0, w: 1 }`
     */
    rotation;
    /**
     * Whether the prefab this component is attached to is [kinematic](https://docs.unity3d.com/ScriptReference/Rigidbody-isKinematic.html) or not.
     *
     * @since v1
     *
     * @example
     * import { NetworkRigidbodyComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new NetworkRigidbodyComponent({ version: componentVersion });
     *
     * const isKinematic = component.isKinematic;
     * // `isKinematic` is `false`
     */
    isKinematic;
    /**
     * Whether the prefab this component is attached to is [sleeping](https://docs.unity3d.com/Manual/RigidbodiesOverview.html) or not.
     *
     * @since v1
     *
     * @example
     * import { NetworkRigidbodyComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new NetworkRigidbodyComponent({ version: componentVersion });
     *
     * const isServerSleeping = component.isServerSleeping;
     * // `isServerSleeping` is `false`
     */
    isServerSleeping;
    /**
     * Velocity of the prefab this component is attached to.
     *
     * @since v1
     *
     * @example
     * import { NetworkRigidbodyComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new NetworkRigidbodyComponent({ version: componentVersion });
     *
     * const velocity = component.velocity;
     * // `velocity` is `{ x: 0, y: 0, z: 0 }`
     */
    velocity;
    /**
     * Angular velocity of the prefab this component is attached to.
     *
     * @since v1
     *
     * @example
     * import { NetworkRigidbodyComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new NetworkRigidbodyComponent({ version: componentVersion });
     *
     * const angularVelocity = component.angularVelocity;
     * // `angularVelocity` is `{ x: 0, y: 0, z: 0 }`
     */
    angularVelocity;
    /**
     * @see [Class: `NetworkRigidbodyComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/NetworkRigidbodyComponent.md)
     * @since v3.0.0
     * @version 1
     *
     * @example
     * import { NetworkRigidbodyComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new NetworkRigidbodyComponent({ version: componentVersion });
     */
    constructor({ version, position, rotation, isKinematic, isServerSleeping, velocity, angularVelocity }) {
        const hash = ComponentHash_js_1.ComponentHash.NetworkRigidbody;
        const name = 'NetworkRigidbody';
        super({ hash, name, version });
        this.position = position ?? { x: 0, y: 0, z: 0 };
        this.rotation = rotation ?? { x: 0, y: 0, z: 0, w: 1 };
        this.isKinematic = isKinematic ?? false;
        this.isServerSleeping = isServerSleeping ?? false;
        this.velocity = velocity ?? { x: 0, y: 0, z: 0 };
        this.angularVelocity = angularVelocity ?? { x: 0, y: 0, z: 0 };
    }
    static fromBinary(reader, version) {
        const props = { version };
        /**
         * @property {Position} position
         * @since v1
         */
        if (props.version >= 1) {
            props.position = {
                x: reader.readFloat(),
                y: reader.readFloat(),
                z: reader.readFloat()
            };
        }
        /**
         * @property {Rotation} rotation
         * @since v1
         */
        if (props.version >= 1) {
            props.rotation = {
                x: reader.readFloat(),
                y: reader.readFloat(),
                z: reader.readFloat(),
                w: reader.readFloat()
            };
        }
        /**
         * @property {boolean} isKinematic
         * @since v1
         */
        if (props.version >= 1) {
            props.isKinematic = reader.readBoolean();
        }
        /**
         * @property {boolean} isServerSleeping
         * @since v1
         */
        if (props.version >= 1) {
            props.isServerSleeping = reader.readBoolean();
        }
        /**
         * @property {Velocity} velocity
         * @since v1
         */
        if (props.version >= 1) {
            props.velocity = {
                x: reader.readFloat(),
                y: reader.readFloat(),
                z: reader.readFloat()
            };
        }
        /**
         * @property {AngularVelocity} angularVelocity
         * @since v1
         */
        if (props.version >= 1) {
            props.angularVelocity = {
                x: reader.readFloat(),
                y: reader.readFloat(),
                z: reader.readFloat()
            };
        }
        return new NetworkRigidbodyComponent(props);
    }
    toBinary(version = this.version) {
        const writer = new BinaryWriter_js_1.BinaryWriter();
        /**
         * @property {Position} position
         * @since v1
         */
        if (version >= 1) {
            writer.writeFloat(this.position.x);
            writer.writeFloat(this.position.y);
            writer.writeFloat(this.position.z);
        }
        /**
         * @property {Rotation} rotation
         * @since v1
         */
        if (version >= 1) {
            writer.writeFloat(this.rotation.x);
            writer.writeFloat(this.rotation.y);
            writer.writeFloat(this.rotation.z);
            writer.writeFloat(this.rotation.w);
        }
        /**
         * @property {boolean} isKinematic
         * @since v1
         */
        if (version >= 1) {
            writer.writeBoolean(this.isKinematic);
        }
        /**
         * @property {boolean} isServerSleeping
         * @since v1
         */
        if (version >= 1) {
            writer.writeBoolean(this.isServerSleeping);
        }
        /**
         * @property {Velocity} velocity
         * @since v1
         */
        if (version >= 1) {
            writer.writeFloat(this.velocity.x);
            writer.writeFloat(this.velocity.y);
            writer.writeFloat(this.velocity.z);
        }
        /**
         * @property {AngularVelocity} angularVelocity
         * @since v1
         */
        if (version >= 1) {
            writer.writeFloat(this.angularVelocity.x);
            writer.writeFloat(this.angularVelocity.y);
            writer.writeFloat(this.angularVelocity.z);
        }
        return writer.flush();
    }
}
exports.NetworkRigidbodyComponent = NetworkRigidbodyComponent;
