"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PhysicalMaterialPartComponent = void 0;
const BinaryWriter_js_1 = require("../BinaryWriter.js");
const ComponentHash_js_1 = require("../types/ComponentHash.js");
const PhysicalMaterialPartHash_js_1 = require("../types/PhysicalMaterialPartHash.js");
const Component_js_1 = require("./Component.js");
/**
 * Controls physical appearance of the `Prefab` it is attached to.
 *
 * @see [Class: `PhysicalMaterialPartComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/PhysicalMaterialPartComponent.md)
 * @since v3.0.0
 * @version 1
 *
 * @example
 * import { PhysicalMaterialPartComponent } from 'att-string-transcoder';
 *
 * const componentVersion = 1;
 * const component = new PhysicalMaterialPartComponent({ version: componentVersion });
 */
class PhysicalMaterialPartComponent extends Component_js_1.Component {
    /**
     * The hash of the physical material of the prefab this component is attached to.
     *
     * @since v1
     *
     * @example
     * import { PhysicalMaterialPartComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new PhysicalMaterialPartComponent({ version: componentVersion });
     *
     * const materialHash = component.materialHash;
     * // `materialHash` is `1`
     */
    materialHash;
    /**
     * @see [Class: `PhysicalMaterialPartComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/PhysicalMaterialPartComponent.md)
     * @since v3.0.0
     * @version 1
     *
     * @example
     * import { PhysicalMaterialPartComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new PhysicalMaterialPartComponent({ version: componentVersion });
     */
    constructor({ version, materialHash }) {
        const hash = ComponentHash_js_1.ComponentHash.PhysicalMaterialPart;
        const name = 'PhysicalMaterialPart';
        super({ hash, name, version });
        this.materialHash = materialHash ?? PhysicalMaterialPartHash_js_1.PhysicalMaterialPartHash.Iron;
    }
    static fromBinary(reader, version) {
        const props = { version };
        /**
         * @property {number} materialHash
         * @since v1
         */
        if (props.version >= 1) {
            props.materialHash = reader.readUnsignedInteger();
        }
        return new PhysicalMaterialPartComponent(props);
    }
    toBinary(version = this.version) {
        const writer = new BinaryWriter_js_1.BinaryWriter();
        /**
         * @property {number} materialHash
         * @since v1
         */
        if (version >= 1) {
            writer.writeUnsignedInteger(this.materialHash);
        }
        return writer.flush();
    }
}
exports.PhysicalMaterialPartComponent = PhysicalMaterialPartComponent;
