"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PopulationSpawnAreaComponent = void 0;
const BinaryWriter_js_1 = require("../BinaryWriter.js");
const ComponentHash_js_1 = require("../types/ComponentHash.js");
const PopulationDefinitionHash_js_1 = require("../types/PopulationDefinitionHash.js");
const Component_js_1 = require("./Component.js");
/**
 * @see [Class: `PopulationSpawnAreaComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/PopulationSpawnAreaComponent.md)
 * @since v3.0.0
 * @version 2
 *
 * @example
 * import { PopulationSpawnAreaComponent } from 'att-string-transcoder';
 *
 * const componentVersion = 2;
 * const component = new PopulationSpawnAreaComponent({ version: componentVersion });
 */
class PopulationSpawnAreaComponent extends Component_js_1.Component {
    /**
     * The hash of the population to spawn in the `SpawnArea` component that is attached to the same prefab this component is attached to.
     *
     * @since v2
     *
     * @example
     * import { PopulationSpawnAreaComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new PopulationSpawnAreaComponent({ version: componentVersion });
     *
     * const definition = component.definition;
     * // `definition` is `30040`
     */
    definition;
    /**
     * @since v2
     *
     * @example
     * import { PopulationSpawnAreaComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new PopulationSpawnAreaComponent({ version: componentVersion });
     *
     * const isPopulationStarted = component.isPopulationStarted;
     * // `isPopulationStarted` is `true`
     */
    isPopulationStarted;
    /**
     * @since v2
     *
     * @example
     * import { PopulationSpawnAreaComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new PopulationSpawnAreaComponent({ version: componentVersion });
     *
     * const children = component.children;
     * // `children` is `[]`
     */
    children;
    /**
     * The maximum amount of spawned prefabs in this population.
     *
     * @since v2
     *
     * @example
     * import { PopulationSpawnAreaComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new PopulationSpawnAreaComponent({ version: componentVersion });
     *
     * const maxPopulation = component.maxPopulation;
     * // `maxPopulation` is `20`
     */
    maxPopulation;
    /**
     * The current amount of spawned prefabs in this population.
     *
     * @since v2
     *
     * @example
     * import { PopulationSpawnAreaComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new PopulationSpawnAreaComponent({ version: componentVersion });
     *
     * const currentPopulation = component.currentPopulation;
     * // `currentPopulation` is `5`
     */
    currentPopulation;
    /**
     * @since v2
     *
     * @example
     * import { PopulationSpawnAreaComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new PopulationSpawnAreaComponent({ version: componentVersion });
     *
     * const checkPopulationTime = component.checkPopulationTime;
     * // `checkPopulationTime` is `852974979330`
     */
    checkPopulationTime;
    /**
     * The amount of spawn locations inside the `SpawnArea` component that is attached to the same prefab this component is attached to.
     *
     * @since v2
     *
     * @example
     * import { PopulationSpawnAreaComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new PopulationSpawnAreaComponent({ version: componentVersion });
     *
     * const numberOfSpawnPoints = component.numberOfSpawnPoints;
     * // `numberOfSpawnPoints` is `40`
     */
    numberOfSpawnPoints;
    /**
     * The amount of spawned prefabs that this population begins with.
     *
     * @since v2
     *
     * @example
     * import { PopulationSpawnAreaComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new PopulationSpawnAreaComponent({ version: componentVersion });
     *
     * const startingPopulation = component.startingPopulation;
     * // `startingPopulation` is `5`
     */
    startingPopulation;
    /**
     * Whether any removed prefabs from this population shouldn't be replaced after the server's respawn timer has elapsed.
     *
     * @since v2
     *
     * @example
     * import { PopulationSpawnAreaComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new PopulationSpawnAreaComponent({ version: componentVersion });
     *
     * const isOneOff = component.isOneOff;
     * // `isOneOff` is `false`
     */
    isOneOff;
    /**
     * @see [Class: `PopulationSpawnAreaComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/PopulationSpawnAreaComponent.md)
     * @since v3.0.0
     * @version 2
     *
     * @example
     * import { PopulationSpawnAreaComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new PopulationSpawnAreaComponent({ version: componentVersion });
     */
    constructor({ version, definition, isPopulationStarted, children, maxPopulation, currentPopulation, checkPopulationTime, numberOfSpawnPoints, startingPopulation, isOneOff }) {
        const hash = ComponentHash_js_1.ComponentHash.PopulationSpawnArea;
        const name = 'PopulationSpawnArea';
        super({ hash, name, version });
        this.definition = definition ?? PopulationDefinitionHash_js_1.PopulationDefinitionHash.StickPopulation;
        this.isPopulationStarted = isPopulationStarted ?? true;
        this.children = children ?? [];
        this.maxPopulation = maxPopulation ?? 20;
        this.currentPopulation = currentPopulation ?? 5;
        this.checkPopulationTime = checkPopulationTime ?? 852974979330;
        this.numberOfSpawnPoints = numberOfSpawnPoints ?? 40;
        this.startingPopulation = startingPopulation ?? 5;
        this.isOneOff = isOneOff ?? false;
    }
    static fromBinary(reader, version) {
        const props = { version };
        /**
         * @property {number} definition
         * @since v2
         */
        if (props.version >= 2) {
            props.definition = reader.readUnsignedInteger();
        }
        /**
         * @property {boolean} isPopulationStarted
         * @since v2
         */
        if (props.version >= 2) {
            props.isPopulationStarted = reader.readBoolean();
        }
        /**
         * @property {PopulationSaveDataChild[]} children
         * @since v2
         */
        if (props.version >= 2) {
            const childrenLength = reader.readUnsignedInteger();
            props.children = [];
            for (let index = 0; index < childrenLength; ++index) {
                /**
                 * @property {boolean} isNull
                 * @since v2
                 */
                const isNull = reader.readBoolean();
                if (isNull) {
                    props.children.push(null);
                    continue;
                }
                props.children.push({
                    /**
                     * @property {number} index
                     * @since v2
                     */
                    index: reader.readSignedInteger(),
                    /**
                     * @property {number} pointIndex
                     * @since v2
                     */
                    pointIndex: reader.readSignedInteger()
                });
            }
        }
        /**
         * @property {number} maxPopulation
         * @since v2
         */
        if (props.version >= 2) {
            props.maxPopulation = reader.readSignedInteger();
        }
        /**
         * @property {number} currentPopulation
         * @since v2
         */
        if (props.version >= 2) {
            props.currentPopulation = reader.readSignedInteger();
        }
        /**
         * @property {number} checkPopulationTime
         * @since v2
         */
        if (props.version >= 2) {
            props.checkPopulationTime = reader.readUnsignedLong();
        }
        /**
         * @property {number} numberOfSpawnPoints
         * @since v2
         */
        if (props.version >= 2) {
            props.numberOfSpawnPoints = reader.readSignedInteger();
        }
        /**
         * @property {number} startingPopulation
         * @since v2
         */
        if (props.version >= 2) {
            props.startingPopulation = reader.readSignedInteger();
        }
        /**
         * @property {boolean} isOneOff
         * @since v2
         */
        if (props.version >= 2) {
            props.isOneOff = reader.readBoolean();
        }
        return new PopulationSpawnAreaComponent(props);
    }
    toBinary(version = this.version) {
        const writer = new BinaryWriter_js_1.BinaryWriter();
        /**
         * @property {number} definition
         * @since v2
         */
        if (version >= 2) {
            writer.writeUnsignedInteger(this.definition);
        }
        /**
         * @property {boolean} isPopulationStarted
         * @since v2
         */
        if (version >= 2) {
            writer.writeBoolean(this.isPopulationStarted);
        }
        /**
         * @property {PopulationSaveDataChild[]} children
         * @since v2
         */
        if (version >= 2) {
            writer.writeUnsignedInteger(this.children.length);
            for (const populationSaveDataChild of this.children) {
                const isNull = populationSaveDataChild === null;
                /**
                 * @property {boolean} isNull
                 * @since v2
                 */
                writer.writeBoolean(isNull);
                if (!isNull) {
                    /**
                     * @property {number} index
                     * @since v2
                     */
                    writer.writeSignedInteger(populationSaveDataChild.index);
                    /**
                     * @property {number} pointIndex
                     * @since v2
                     */
                    writer.writeSignedInteger(populationSaveDataChild.pointIndex);
                }
            }
        }
        /**
         * @property {number} maxPopulation
         * @since v2
         */
        if (version >= 2) {
            writer.writeSignedInteger(this.maxPopulation);
        }
        /**
         * @property {number} currentPopulation
         * @since v2
         */
        if (version >= 2) {
            writer.writeSignedInteger(this.currentPopulation);
        }
        /**
         * @property {number} checkPopulationTime
         * @since v2
         */
        if (version >= 2) {
            writer.writeUnsignedLong(this.checkPopulationTime);
        }
        /**
         * @property {number} numberOfSpawnPoints
         * @since v2
         */
        if (version >= 2) {
            writer.writeSignedInteger(this.numberOfSpawnPoints);
        }
        /**
         * @property {number} startingPopulation
         * @since v2
         */
        if (version >= 2) {
            writer.writeSignedInteger(this.startingPopulation);
        }
        /**
         * @property {boolean} isOneOff
         * @since v2
         */
        if (version >= 2) {
            writer.writeBoolean(this.isOneOff);
        }
        return writer.flush();
    }
}
exports.PopulationSpawnAreaComponent = PopulationSpawnAreaComponent;
