"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StatManagerComponent = void 0;
const BinaryWriter_js_1 = require("../BinaryWriter.js");
const ComponentHash_js_1 = require("../types/ComponentHash.js");
const Component_js_1 = require("./Component.js");
/**
 * @see [Class: `StatManagerComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/StatManagerComponent.md)
 * @since v3.0.0
 * @version 2
 *
 * @example
 * import { StatManagerComponent } from 'att-string-transcoder';
 *
 * const componentVersion = 2;
 * const component = new StatManagerComponent({ version: componentVersion });
 */
class StatManagerComponent extends Component_js_1.Component {
    /**
     * @since v2
     *
     * @example
     * import { StatManagerComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new StatManagerComponent({ version: componentVersion });
     *
     * const stats = component.stats;
     * // `stats` is `[]`
     */
    stats;
    /**
     * @since v2
     *
     * @example
     * import { StatManagerComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new StatManagerComponent({ version: componentVersion });
     *
     * const modifiers = component.modifiers;
     * // `modifiers` is `[]`
     */
    modifiers;
    /**
     * @since v2
     *
     * @example
     * import { StatManagerComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new StatManagerComponent({ version: componentVersion });
     *
     * const indirectStatModifiers = component.indirectStatModifiers;
     * // `indirectStatModifiers` is `[]`
     */
    indirectStatModifiers;
    /**
     * @see [Class: `StatManagerComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/StatManagerComponent.md)
     * @since v3.0.0
     * @version 2
     *
     * @example
     * import { StatManagerComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new StatManagerComponent({ version: componentVersion });
     */
    constructor({ version, stats, modifiers, indirectStatModifiers }) {
        const hash = ComponentHash_js_1.ComponentHash.StatManager;
        const name = 'StatManager';
        super({ hash, name, version });
        this.stats = stats ?? [];
        this.modifiers = modifiers ?? [];
        this.indirectStatModifiers = indirectStatModifiers ?? [];
    }
    static fromBinary(reader, version) {
        const props = { version };
        /**
         * @property {Stat[]} stats
         * @since v2
         */
        if (props.version >= 2) {
            const statsLength = reader.readUnsignedInteger();
            props.stats = [];
            for (let index = 0; index < statsLength; ++index) {
                /**
                 * @property {boolean} isNull
                 * @since v2
                 */
                const isNull = reader.readBoolean();
                if (isNull) {
                    props.stats.push(null);
                    continue;
                }
                props.stats.push({
                    /**
                     * @property {number} hash
                     * @since v2
                     */
                    hash: reader.readUnsignedInteger(),
                    /**
                     * @property {float} baseFlat
                     * @since v2
                     */
                    baseFlat: reader.readFloat()
                });
            }
        }
        /**
         * @property {TimedModifier[]} modifiers
         * @since v2
         */
        if (props.version >= 2) {
            const modifiersLength = reader.readUnsignedInteger();
            props.modifiers = [];
            for (let index = 0; index < modifiersLength; ++index) {
                /**
                 * @property {boolean} isNull
                 * @since v2
                 */
                const isNull = reader.readBoolean();
                if (isNull) {
                    props.modifiers.push(null);
                    continue;
                }
                props.modifiers.push({
                    /**
                     * @property {number} hash
                     * @since v2
                     */
                    hash: reader.readUnsignedInteger(),
                    /**
                     * @property {float} value
                     * @since v2
                     */
                    value: reader.readFloat(),
                    /**
                     * @property {boolean} isMultiplier
                     * @since v2
                     */
                    isMultiplier: reader.readBoolean(),
                    /**
                     * @property {number} time
                     * @since v2
                     */
                    time: reader.readUnsignedLong()
                });
            }
        }
        /**
         * @property {IndirectStatModifier[]} indirectStatModifiers
         * @since v2
         */
        if (props.version >= 2) {
            const indirectStatModifiersLength = reader.readUnsignedInteger();
            props.indirectStatModifiers = [];
            for (let index = 0; index < indirectStatModifiersLength; ++index) {
                /**
                 * @property {boolean} isNull
                 * @since v2
                 */
                const isNull = reader.readBoolean();
                if (isNull) {
                    props.indirectStatModifiers.push(null);
                    continue;
                }
                /**
                 * @property {number} hash
                 * @since v2
                 */
                const hash = reader.readUnsignedInteger();
                /**
                 * @property {number} time
                 * @since v2
                 */
                const time = reader.readUnsignedLong();
                /**
                 * @property {IndirectModifierSaveData[]} indirectModifiersSaveData
                 * @since v2
                 */
                const indirectModifiersSaveDataLength = reader.readUnsignedInteger();
                const indirectModifiersSaveData = [];
                for (let indirectModifiersSaveDataIndex = 0; indirectModifiersSaveDataIndex < indirectModifiersSaveDataLength; ++indirectModifiersSaveDataIndex) {
                    indirectModifiersSaveData.push({
                        /**
                         * @property {number} valueOverDurationHash
                         * @since v2
                         */
                        valueOverDurationHash: reader.readUnsignedInteger(),
                        /**
                         * @property {float} baseValue
                         * @since v2
                         */
                        baseValue: reader.readFloat(),
                        /**
                         * @property {number} duration
                         * @since v2
                         */
                        duration: reader.readUnsignedInteger(),
                        /**
                         * @property {number} tick
                         * @since v2
                         */
                        tick: reader.readUnsignedInteger()
                    });
                }
                props.indirectStatModifiers.push({
                    hash,
                    time,
                    modifiers: indirectModifiersSaveData
                });
            }
        }
        return new StatManagerComponent(props);
    }
    toBinary(version = this.version) {
        const writer = new BinaryWriter_js_1.BinaryWriter();
        /**
         * @property {Stat[]} stats
         * @since v2
         */
        if (version >= 2) {
            writer.writeUnsignedInteger(this.stats.length);
            for (const stat of this.stats) {
                const isNull = stat === null;
                /**
                 * @property {boolean} isNull
                 * @since v2
                 */
                writer.writeBoolean(isNull);
                if (!isNull) {
                    /**
                     * @property {number} hash
                     * @since v2
                     */
                    writer.writeUnsignedInteger(stat.hash);
                    /**
                     * @property {float} baseFlat
                     * @since v2
                     */
                    writer.writeFloat(stat.baseFlat);
                }
            }
        }
        /**
         * @property {TimedModifier[]} modifiers
         * @since v2
         */
        if (version >= 2) {
            writer.writeUnsignedInteger(this.modifiers.length);
            for (const modifier of this.modifiers) {
                const isNull = modifier === null;
                /**
                 * @property {boolean} isNull
                 * @since v2
                 */
                writer.writeBoolean(isNull);
                if (!isNull) {
                    /**
                     * @property {number} hash
                     * @since v2
                     */
                    writer.writeUnsignedInteger(modifier.hash);
                    /**
                     * @property {float} value
                     * @since v2
                     */
                    writer.writeFloat(modifier.value);
                    /**
                     * @property {boolean} isMultiplier
                     * @since v2
                     */
                    writer.writeBoolean(modifier.isMultiplier);
                    /**
                     * @property {number} time
                     * @since v2
                     */
                    writer.writeUnsignedLong(modifier.time);
                }
            }
        }
        /**
         * @property {IndirectStatModifier[]} indirectStatModifiers
         * @since v2
         */
        if (version >= 2) {
            writer.writeUnsignedInteger(this.indirectStatModifiers.length);
            for (const indirectStatModifier of this.indirectStatModifiers) {
                const isNull = indirectStatModifier === null;
                /**
                 * @property {boolean} isNull
                 * @since v2
                 */
                writer.writeBoolean(isNull);
                if (!isNull) {
                    /**
                     * @property {number} hash
                     * @since v2
                     */
                    writer.writeUnsignedInteger(indirectStatModifier.hash);
                    /**
                     * @property {number} time
                     * @since v2
                     */
                    writer.writeUnsignedLong(indirectStatModifier.time);
                    /**
                     * @property {IndirectModifierSaveData[]} indirectModifiersSaveData
                     * @since v2
                     */
                    writer.writeUnsignedInteger(indirectStatModifier.modifiers.length);
                    for (const modifier of indirectStatModifier.modifiers) {
                        /**
                         * @property {number} valueOverDurationHash
                         * @since v2
                         */
                        writer.writeUnsignedInteger(modifier.valueOverDurationHash);
                        /**
                         * @property {float} baseValue
                         * @since v2
                         */
                        writer.writeFloat(modifier.baseValue);
                        /**
                         * @property {number} duration
                         * @since v2
                         */
                        writer.writeUnsignedInteger(modifier.duration);
                        /**
                         * @property {number} tick
                         * @since v2
                         */
                        writer.writeUnsignedInteger(modifier.tick);
                    }
                }
            }
        }
        return writer.flush();
    }
}
exports.StatManagerComponent = StatManagerComponent;
