"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WoodcutTreeComponent = void 0;
const BinaryWriter_js_1 = require("../BinaryWriter.js");
const ComponentHash_js_1 = require("../types/ComponentHash.js");
const SpeciesHash_js_1 = require("../types/SpeciesHash.js");
const Component_js_1 = require("./Component.js");
/**
 * @see [Class: `WoodcutTreeComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/WoodcutTreeComponent.md)
 * @since v3.0.0
 * @version 3
 *
 * @example
 * import { WoodcutTreeComponent } from 'att-string-transcoder';
 *
 * const componentVersion = 3;
 * const component = new WoodcutTreeComponent({ version: componentVersion });
 */
class WoodcutTreeComponent extends Component_js_1.Component {
    /**
     * @since v3
     *
     * @example
     * import { WoodcutTreeComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 3;
     * const component = new WoodcutTreeComponent({ version: componentVersion });
     *
     * const presetHash = component.presetHash;
     * // `presetHash` is `0`
     */
    presetHash;
    /**
     * The hash of the species of the tree prefab this component is attached to.
     *
     * @since v3
     *
     * @example
     * import { WoodcutTreeComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 3;
     * const component = new WoodcutTreeComponent({ version: componentVersion });
     *
     * const speciesHash = component.speciesHash;
     * // `speciesHash` is `11232`
     */
    speciesHash;
    /**
     * @see [Class: `WoodcutTreeComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/WoodcutTreeComponent.md)
     * @since v3.0.0
     * @version 3
     *
     * @example
     * import { WoodcutTreeComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 3;
     * const component = new WoodcutTreeComponent({ version: componentVersion });
     */
    constructor({ version, presetHash, speciesHash }) {
        const hash = ComponentHash_js_1.ComponentHash.WoodcutTree;
        const name = 'WoodcutTree';
        super({ hash, name, version });
        this.presetHash = presetHash ?? 0;
        this.speciesHash = speciesHash ?? SpeciesHash_js_1.SpeciesHash.Oak;
    }
    static fromBinary(reader, version) {
        const props = { version };
        /**
         * @property {number} presetHash
         * @since v3
         */
        if (props.version >= 3) {
            props.presetHash = reader.readUnsignedInteger();
        }
        /**
         * @property {number} speciesHash
         * @since v3
         */
        if (props.version >= 3) {
            props.speciesHash = reader.readUnsignedInteger();
        }
        return new WoodcutTreeComponent(props);
    }
    toBinary(version = this.version) {
        const writer = new BinaryWriter_js_1.BinaryWriter();
        /**
         * @property {number} presetHash
         * @since v3
         */
        /**
         * @property {number} presetHash
         * @since v3
         */
        if (version >= 3) {
            writer.writeUnsignedInteger(this.presetHash);
        }
        /**
         * @property {number} speciesHash
         * @since v3
         */
        if (version >= 3) {
            writer.writeUnsignedInteger(this.speciesHash);
        }
        return writer.flush();
    }
}
exports.WoodcutTreeComponent = WoodcutTreeComponent;
