"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.readComponents = void 0;
const constants_js_1 = require("../constants.js");
const ComponentHash_js_1 = require("../types/ComponentHash.js");
const isUnknownComponent_js_1 = require("./isUnknownComponent.js");
const readComponent_js_1 = require("./readComponent.js");
/**
 * Reads all components from the given `BinaryReader` and returns a `PrefabComponents` object.
 *
 * @example
 * import { BinaryReader, readComponents } from 'att-string-transcoder';
 *
 * const reader = new BinaryReader('...');
 *
 * const versions = new Map<number, number>([
 *   [ComponentHash.NetworkRigidbody, 1],
 *   [ComponentHash.PhysicalMaterialPart, 1],
 *   [ComponentHash.Pickup, 2]
 *   // etc...
 * ]);
 *
 * const component = readComponents(reader, versions);
 */
function readComponents(reader, versions) {
    const components = Object.assign({}, { Unknown: [] });
    const isLooping = true;
    while (isLooping) {
        const hash = reader.readUnsignedInteger();
        if (hash === constants_js_1.terminatorHash)
            break;
        const name = ComponentHash_js_1.ComponentHash[hash] ?? 'Unknown';
        const version = versions?.get(hash) ?? constants_js_1.latestSupportedComponentVersions.get(hash);
        if (typeof version === 'undefined') {
            throw new Error(`Could not determine version of "${name}" (${hash}) component. A SaveString with chunk versioning pairs is required to read this data.`);
        }
        const component = (0, readComponent_js_1.readComponent)(reader, hash, name, version);
        if ((0, isUnknownComponent_js_1.isUnknownComponent)(component)) {
            components.Unknown.push(component);
        }
        else {
            components[name] = component;
        }
    }
    return components;
}
exports.readComponents = readComponents;
