"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.readEntities = void 0;
const Entity_js_1 = require("../Entity.js");
const constants_js_1 = require("../constants.js");
const ATTPrefabs_js_1 = require("../types/ATTPrefabs.js");
/**
 * Reads all entities from the given `BinaryReader` and returns a `PrefabEntities` object.
 *
 * @example
 * import { BinaryReader, readEntities, type ATTPrefabName } from 'att-string-transcoder';
 *
 * const reader = new BinaryReader('...');
 * const prefabName = 'Handle_Short' as ATTPrefabName;
 *
 * const componentVersions = new Map<number, number>([
 *   [ComponentHash.NetworkRigidbody, 1],
 *   [ComponentHash.PhysicalMaterialPart, 1],
 *   [ComponentHash.Pickup, 2]
 *   // etc...
 * ]);
 *
 * const component = readEntities(reader, prefabName, componentVersions);
 */
function readEntities(reader, prefabName, componentVersions) {
    const entities = {};
    const isLooping = true;
    while (isLooping) {
        const hash = reader.readUnsignedInteger();
        if (hash === constants_js_1.terminatorHash)
            break;
        /* Get the entity's data length and ignore it. */
        reader.readUnsignedInteger();
        const entity = Object.values(ATTPrefabs_js_1.ATTPrefabs[prefabName].embedded).find(attPrefabEntity => attPrefabEntity.hash === hash);
        const name = entity?.name ?? 'Unknown';
        const key = `${name}_${hash}`;
        entities[key] = Entity_js_1.Entity.fromBinary(reader, {
            hash,
            key: name === 'Unknown' ? name : key,
            componentVersions
        });
    }
    return entities;
}
exports.readEntities = readEntities;
