"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeComponents = void 0;
const constants_js_1 = require("../constants.js");
/**
 * Writes given components to the given `BinaryWriter`.
 *
 * @example
 * import { BinaryWriter, writeComponents } from 'att-string-transcoder';
 *
 * const writer = new BinaryWriter();
 *
 * const components = {
 *   NetworkRigidbody: new NetworkRigidbody({ version: 1 }),
 *   Unknown: []
 * };
 *
 * const versions = new Map<number, number>([
 *   [ComponentHash.NetworkRigidbody, 1],
 *   [ComponentHash.PhysicalMaterialPart, 1],
 *   [ComponentHash.Pickup, 2]
 *   // etc...
 * ]);
 *
 * writeComponents(writer, components, versions);
 */
function writeComponents(writer, components, versions) {
    for (const component of Object.values(components)) {
        if (Array.isArray(component)) {
            for (const unknownComponent of component) {
                /**
                 * @property {Component} unknownComponent
                 */
                unknownComponent.write(writer, versions.get(unknownComponent.hash));
            }
        }
        else {
            /**
             * @property {Component} unknownComponent
             */
            component.write(writer, versions.get(component.hash));
        }
    }
    /**
     * @property {number} terminatorHash
     */
    writer.writeUnsignedInteger(constants_js_1.terminatorHash);
}
exports.writeComponents = writeComponents;
