import { latestSupportedComponentVersions, terminatorHash } from '../constants.js';
import { ComponentHash } from '../types/ComponentHash.js';
import { isUnknownComponent } from './isUnknownComponent.js';
import { readComponent } from './readComponent.js';
/**
 * Reads all components from the given `BinaryReader` and returns a `PrefabComponents` object.
 *
 * @example
 * import { BinaryReader, readComponents } from 'att-string-transcoder';
 *
 * const reader = new BinaryReader('...');
 *
 * const versions = new Map<number, number>([
 *   [ComponentHash.NetworkRigidbody, 1],
 *   [ComponentHash.PhysicalMaterialPart, 1],
 *   [ComponentHash.Pickup, 2]
 *   // etc...
 * ]);
 *
 * const component = readComponents(reader, versions);
 */
export function readComponents(reader, versions) {
    const components = Object.assign({}, { Unknown: [] });
    const isLooping = true;
    while (isLooping) {
        const hash = reader.readUnsignedInteger();
        if (hash === terminatorHash)
            break;
        const name = ComponentHash[hash] ?? 'Unknown';
        const version = versions?.get(hash) ?? latestSupportedComponentVersions.get(hash);
        if (typeof version === 'undefined') {
            throw new Error(`Could not determine version of "${name}" (${hash}) component. A SaveString with chunk versioning pairs is required to read this data.`);
        }
        const component = readComponent(reader, hash, name, version);
        if (isUnknownComponent(component)) {
            components.Unknown.push(component);
        }
        else {
            components[name] = component;
        }
    }
    return components;
}
