import type { BinaryString } from './types/BinaryString.js';
import type { SaveString } from './types/SaveString.js';
export type BinaryDataOptions = {
    signedIntegerOffset?: number;
};
export type UnpackFloatOptions = {
    bufferOffset?: number;
    byteLength?: number;
    isLittleEndian?: boolean;
    mantissaLength?: number;
};
/**
 * Deals with data types as understood by _A Township Tale_ game servers.
 *
 * @see [Class: `BinaryData`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/BinaryData.md)
 * @since v3.0.0
 *
 * @example
 * import { BinaryData } from 'att-string-transcoder';
 *
 * const data = new BinaryData('11110111001100010000');
 * // or
 * const data = BinaryData.fromBoolean(true);
 * // or
 * const data = BinaryData.fromChar('x');
 * // or
 * const data = BinaryData.fromFloat(0.69);
 * // or
 * const data = BinaryData.fromNumber(1337);
 * // or
 * const data = BinaryData.fromSignedInteger(-420);
 * // or
 * const data = BinaryData.fromUnsignedInteger(69420);
 * // or
 * const data = BinaryData.fromUnsignedIntegerArray([69420, 133769]);
 * // or
 * const data = BinaryData.fromUnsignedLong(133742069);
 * // or
 * const data = BinaryData.fromUnsignedShort(420);
 */
export declare class BinaryData {
    private readonly binary;
    private readonly signedIntegerOffset;
    /**
     * Creates a data structure from the given binary `data` and lets you express it as other data
     * types. You may optionally provide additional options in the second argument.
     *
     * @see [Class: `BinaryData`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/BinaryData.md)
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = new BinaryData('11110111001100010000', {
     *   signedIntegerOffset: 2147483648
     * });
     */
    constructor(binary: string, options?: BinaryDataOptions);
    /**
     * Creates a BinaryData instance from the given boolean.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = BinaryData.fromBoolean(true);
     */
    static fromBoolean(boolean: boolean, options?: BinaryDataOptions): BinaryData;
    /**
     * Creates a BinaryData instance from the given character.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = BinaryData.fromChar('x');
     */
    static fromChar(char: string, options?: BinaryDataOptions): BinaryData;
    /**
     * Creates a BinaryData instance from the given floating point number.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = BinaryData.fromFloat(0.69);
     */
    static fromFloat(float: number, options?: BinaryDataOptions): BinaryData;
    /**
     * Creates a BinaryData instance from the given number.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = BinaryData.fromNumber(1337);
     */
    static fromNumber(number: number, options?: BinaryDataOptions): BinaryData;
    /**
     * Creates a BinaryData instance from the given signed integer.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = BinaryData.fromSignedInteger(-420);
     */
    static fromSignedInteger(signedInteger: number, options?: BinaryDataOptions): BinaryData;
    /**
     * Creates a BinaryData instance from the given unsigned integer.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = BinaryData.fromUnsignedInteger(69420);
     */
    static fromUnsignedInteger(unsignedInteger: number, options?: BinaryDataOptions): BinaryData;
    /**
     * Creates a BinaryData instance from the given comma-separated array of unsigned integers.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = BinaryData.fromUnsignedIntegerArray([69420, 133769]);
     */
    static fromUnsignedIntegerArray(unsignedIntegerArray: number[], options?: BinaryDataOptions): BinaryData;
    /**
     * Creates a BinaryData instance from the given unsigned long integer.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = BinaryData.fromUnsignedLong(133742069);
     */
    static fromUnsignedLong(unsignedLong: number, options?: BinaryDataOptions): BinaryData;
    /**
     * Creates a BinaryData instance from the given unsigned short integer.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = BinaryData.fromUnsignedShort(420);
     */
    static fromUnsignedShort(unsignedShort: number, options?: BinaryDataOptions): BinaryData;
    /**
     * Confirms whether `data` is a string of binary data, i.e. a string of zeroes and ones.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const maybeBinaryString = '11110111001100010000';
     * // `maybeBinaryString` is of type `string`
     *
     * if (BinaryData.isBinaryString(maybeBinaryString)) {
     *   // `maybeBinaryString` is of type `BinaryString` inside this closure
     * }
     */
    static isBinaryString(data: string): data is BinaryString;
    /**
     * Confirms whether `data` is a string of unsigned integers and chunk versioning pairs.
     *
     * This is also a type assertion function, so if the passed `string` is asserted as a
     * `BinaryString`, it will be considered as such after calling this function.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const maybeSaveString = '...';
     * // `maybeSaveString` is of type `string`
     *
     * if (BinaryData.isSaveString(maybeSaveString)) {
     *   // `maybeSaveString` is of type `SaveString` inside this closure
     * }
     */
    static isSaveString(data: string): data is SaveString;
    /**
     * Converts a JavaScript floating point number into an IEEE754 integer.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const packedFloat = BinaryData.packFloat(1337.42069);
     * // `packedFloat` is `1151806838`
     */
    static packFloat(number: number): number;
    /**
     * Returns the binary data as a `BinaryString`.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = new BinaryData('1111011100110001000');
     *
     * const binary = data.toBinaryString();
     * // `binary` is `'1111011100110001000'`
     */
    toBinaryString(): BinaryString;
    /**
     * Returns the boolean represenation of the binary data.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = new BinaryData('1');
     *
     * const boolean = data.toBoolean();
     * // `boolean` is `true`
     */
    toBoolean(): boolean;
    /**
     * Returns the character represenation of the binary data.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = new BinaryData('01000001');
     *
     * const char = data.toChar();
     * // `char` is `'A'`
     */
    toChar(): string;
    /**
     * Returns the floating point number represenation of the binary data.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = new BinaryData('01000100101001110010110101110110');
     *
     * const float = data.toFloat();
     * // `float` is approximately `1337.42069`
     */
    toFloat(): number;
    /**
     * Returns the numeric represenation of the binary data.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = new BinaryData('10100111001');
     *
     * const number = data.toNumber();
     * // `number` is `1337`
     */
    toNumber(): number;
    /**
     * Returns the signed integer representation of the binary data.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = new BinaryData('01010110100111110101010000000111');
     *
     * const signedInteger = data.toSignedInteger();
     * // `signedInteger` is `-694201337`
     */
    toSignedInteger(): number;
    /**
     * Returns the unsigned integer representation of the binary data.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = new BinaryData('00000111111110001011110111110101');
     *
     * const unsignedInteger = data.toUnsignedInteger();
     * // `unsignedInteger` is `133742069`
     */
    toUnsignedInteger(): number;
    /**
     * Returns the binary data as a comma-separated array of unsigned integers.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = new BinaryData('000000000000000000000101001110010000000000000000000000011010010000000000000000000000000001000101');
     *
     * const unsignedIntegers = data.toUnsignedIntegerArray();
     * // `unsignedIntegers` is `[1337, 420, 69]`
     */
    toUnsignedIntegerArray(): number[];
    /**
     * Returns the unsigned long integer representation of the binary data.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = new BinaryData('0000011111111000101111011111010100000000000000000000000000000000');
     *
     * const unsignedLongInteger = data.toUnsignedLong();
     * // `unsignedLongInteger` is `133742069`
     */
    toUnsignedLong(): number;
    /**
     * Returns the unsigned short integer representation of the binary data.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const data = new BinaryData('0000010100111001');
     *
     * const unsignedShortInteger = data.toUnsignedShort();
     * // `unsignedShortInteger` is `1337`
     */
    toUnsignedShort(): number;
    /**
     * Converts an IEEE754 integer into a JavaScript floating point number.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryData } from 'att-string-transcoder';
     *
     * const unpackedFloat = BinaryData.unpackFloat(1151806838);
     * // `unpackedFloat` is approximately `1337.42069`
     */
    static unpackFloat(unsignedInteger: number, options?: UnpackFloatOptions): number;
}
