import type { BinaryString } from './types/BinaryString.js';
/**
 * Handles the controlled storing of binary data for ATT save strings.
 *
 * @see [Class: `BinaryWriter`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/BinaryWriter.md)
 * @since v3.0.0
 *
 * @example
 * import { BinaryWriter } from 'att-string-transcoder';
 *
 * const writer = new BinaryWriter();
 *
 * writer.writeBoolean(true);
 *
 * const output = writer.flush();
 * // `output` is `'1'`
 */
export declare class BinaryWriter {
    private data;
    /**
     * Creates a writer to store information using various write methods.
     *
     * @see [Class: `BinaryWriter`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/BinaryWriter.md)
     * @since v3.0.0
     *
     * @example
     * import { BinaryWriter } from 'att-string-transcoder';
     *
     * const writer = new BinaryWriter();
     */
    constructor();
    /**
     * Returns the stored binary string and resets the BinaryWriter's internal memory.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryWriter } from 'att-string-transcoder';
     *
     * const writer = new BinaryWriter();
     *
     * writer.writeBoolean(true); // internal memory is `'1'`
     * writer.writeBoolean(true); // internal memory is `'11'`
     * writer.writeBoolean(true); // internal memory is `'111'`
     *
     * const output = writer.flush(); // internal memory is `''`
     * // `output` is `'111'`
     */
    flush(): BinaryString;
    /**
     * Writes raw binary data into BinaryWriter's internal memory.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryWriter } from 'att-string-transcoder';
     *
     * const writer = new BinaryWriter();
     *
     * writer.writeBits('11110111001100010000');
     */
    writeBits(data: string): void;
    /**
     * Writes a boolean into BinaryWriter's internal memory.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryWriter } from 'att-string-transcoder';
     *
     * const writer = new BinaryWriter();
     *
     * writer.writeBoolean(true);
     */
    writeBoolean(boolean: boolean): void;
    /**
     * Writes a character into BinaryWriter's internal memory.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryWriter } from 'att-string-transcoder';
     *
     * const writer = new BinaryWriter();
     *
     * writer.writeChar('x');
     */
    writeChar(char: string): void;
    /**
     * Writes a floating point number into BinaryWriter's internal memory.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryWriter } from 'att-string-transcoder';
     *
     * const writer = new BinaryWriter();
     *
     * writer.writeFloat(0.69);
     */
    writeFloat(float: number): void;
    /**
     * Writes a signed integer into BinaryWriter's internal memory.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryWriter } from 'att-string-transcoder';
     *
     * const writer = new BinaryWriter();
     *
     * writer.writeSignedInteger(-420);
     */
    writeSignedInteger(signedInteger: number): void;
    /**
     * Writes a string into BinaryWriter's internal memory.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryWriter } from 'att-string-transcoder';
     *
     * const writer = new BinaryWriter();
     *
     * writer.writeString('Hello, world!');
     */
    writeString(string: string): void;
    /**
     * Writes an unsigned integer into BinaryWriter's internal memory.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryWriter } from 'att-string-transcoder';
     *
     * const writer = new BinaryWriter();
     *
     * writer.writeUnsignedInteger(69420);
     */
    writeUnsignedInteger(unsignedInteger: number): void;
    /**
     * Writes an unsigned long integer into BinaryWriter's internal memory.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryWriter } from 'att-string-transcoder';
     *
     * const writer = new BinaryWriter();
     *
     * writer.writeUnsignedLong(133742069);
     */
    writeUnsignedLong(unsignedLong: number): void;
    /**
     * Writes an unsigned short integer into BinaryWriter's internal memory.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryWriter } from 'att-string-transcoder';
     *
     * const writer = new BinaryWriter();
     *
     * writer.writeUnsignedShort(1337);
     */
    writeUnsignedShort(unsignedShort: number): void;
}
