import type { BinaryReader } from './BinaryReader.js';
import type { Component } from './components/Component.js';
import type { ATTPrefabName } from './types/ATTPrefabName.js';
import type { BinaryString } from './types/BinaryString.js';
import type { EntityKey } from './types/EntityKey.js';
import type { PrefabComponents } from './types/PrefabComponents.js';
import type { SupportedPrefabComponents } from './types/SupportedPrefabComponents.js';
import type { UnknownPrefabComponents } from './types/UnknownPrefabComponents.js';
import type { UnsupportedPrefabComponents } from './types/UnsupportedPrefabComponents.js';
import { BinaryWriter } from './BinaryWriter.js';
import { ComponentHash } from './types/ComponentHash.js';
import { PhysicalMaterialPartHash } from './types/PhysicalMaterialPartHash.js';
export type EntityProps = {
    hash?: number | undefined;
    isAlive?: boolean | undefined;
    components?: (SupportedPrefabComponents & UnsupportedPrefabComponents & Partial<UnknownPrefabComponents>) | undefined;
};
type EntityFromBinaryProps = {
    key: string;
    hash: number;
    componentVersions?: Map<number, number> | undefined;
};
/**
 * An "item" inside a prefab composition. The `Entity` can have its own components to alter its
 * behaviour. A child prefab of the composition may attach to an `Entity`, forming an hierarchical
 * structure inside the composition. This is primarily used in _A Township Tale_'s crafting system.
 *
 * @see [Class: `Entity`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/Entity.md)
 * @since v3.0.0
 *
 * @example
 * import { Entity } from 'att-string-transcoder';
 *
 * const torchFireEntity = new Entity<'Torch'>('Fire_30100');
 * // or
 * const torchFireEntity = Entity.fromBinary<'Torch'>(myBinaryString);
 */
export declare class Entity<TPrefabName extends ATTPrefabName> {
    /**
     * The hash of the entity.
     *
     * @since v3.0.0
     *
     * @example
     * import { Entity } from 'att-string-transcoder';
     *
     * const entity = new Entity<'Torch'>('Fire_30100');
     *
     * const hash = entity.hash;
     * // `hash` is `30100`
     */
    hash: number;
    /**
     * The name of the entity.
     *
     * @since v3.0.0
     *
     * @example
     * import { Entity } from 'att-string-transcoder';
     *
     * const entity = new Entity<'Torch'>('Fire_30100');
     *
     * const name = entity.name;
     * // `name` is `'Fire'`
     */
    name: string;
    /**
     * The alive state of the entity.
     *
     * @since v3.0.0
     *
     * @example
     * import { Entity } from 'att-string-transcoder';
     *
     * const entity = new Entity<'Torch'>('Fire_30100');
     *
     * const isAlive = entity.isAlive;
     * // `isAlive` is `true`
     */
    isAlive: boolean;
    /**
     * A map of the stored components.
     *
     * Provides access to the components stored in this entity. Components are keyed to their
     * respective names, unless its data contained an unrecognised hash. In that case, the component
     * will be stored in an array under the `Unknown` key.
     *
     * @since v3.0.0
     *
     * @example
     * import { Entity } from 'att-string-transcoder';
     *
     * const entity = new Entity<'Torch'>('Fire_30100');
     *
     * const components = entity.components;
     * // `components` is `{ Unknown: [] }`
     */
    components: PrefabComponents;
    /**
     * Creates a new `Entity` object configured with the passed in configuration.
     *
     * @see [Class: `Entity`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/Entity.md)
     * @since v3.0.0
     *
     * @example
     * import { Entity } from 'att-string-transcoder';
     *
     * const torchFireEntity = new Entity<'Torch'>('Fire_30100', {
     *   hash: 30100,
     *   isAlive: true,
     *   components: {}
     * });
     */
    constructor(key: EntityKey<TPrefabName>, { hash, isAlive, components }?: EntityProps);
    /**
     * Adds a `Component` to the entity. Will override any existing component with that name.
     *
     * @since v3.1.0
     *
     * @example
     * import { Entity, PhysicalMaterialPartComponent, PhysicalMaterialPartHash } from 'att-string-transcoder';
     *
     * const entity = new Entity<'Standard_Side_Pouch_Attachment'>('standard_sidePouch_backPin_L1_7968');
     * const component = new PhysicalMaterialPartComponent({ version: 1, materialHash: PhysicalMaterialPartHash.Iron });
     *
     * entity.addComponent(component);
     */
    addComponent(component: Component): this;
    /**
     * Reads the binary string data and returns an instantiated entity.
     *
     * @since v3.0.0
     *
     * @example
     * import { Entity } from 'att-string-transcoder';
     *
     * const torchFireEntity = Entity.fromBinary<'Torch'>(binaryReader, { key, hash });
     */
    static fromBinary<TPrefabName extends ATTPrefabName>(reader: BinaryReader, { key, hash, componentVersions }: EntityFromBinaryProps): Entity<TPrefabName>;
    /**
     * Gets the entity's physical material.
     *
     * @since v3.1.0
     *
     * @example
     * import { Entity, PhysicalMaterialPartHash } from 'att-string-transcoder';
     *
     * const entity = new Entity<'Standard_Side_Pouch_Attachment'>('standard_sidePouch_backPin_L1_7968');
     *
     * const materialHash = entity.getMaterial();
     * const materialName = PhysicalMaterialPartHash[materialHash];
     */
    getMaterial(): PhysicalMaterialPartHash;
    /**
     * Removes all components on this entity.
     *
     * @since v3.1.0
     *
     * @example
     * import { Entity } from 'att-string-transcoder';
     *
     * const entity = new Entity<'Standard_Side_Pouch_Attachment'>('standard_sidePouch_backPin_L1_7968');
     *
     * entity.removeAllComponents();
     */
    removeAllComponents(): this;
    /**
     * Removes the specified component from this entity.
     *
     * @since v3.1.0
     *
     * @example
     * import { ComponentHash, Entity } from 'att-string-transcoder';
     *
     * const entity = new Entity<'Standard_Side_Pouch_Attachment'>('standard_sidePouch_backPin_L1_7968');
     *
     * entity.removeComponent(ComponentHash.NetworkRigidbody);
     * // or
     * entity.removeComponent('NetworkRigidbody');
     */
    removeComponent(componentHash: ComponentHash): this;
    removeComponent(componentName: keyof Omit<PrefabComponents, 'Unknown'>): this;
    /**
     * Sets the entity's physical material. This can change both its appearance and other qualities such
     * as durability, damage, heat retention and weight.
     *
     * @since v3.1.0
     *
     * @example
     * import { Entity, PhysicalMaterialPartHash } from 'att-string-transcoder';
     *
     * const entity = new Entity<'Standard_Side_Pouch_Attachment'>('standard_sidePouch_backPin_L1_7968');
     *
     * entity.setMaterial(PhysicalMaterialPartHash.Mythril);
     * // or
     * entity.setMaterial('Mythril');
     */
    setMaterial(materialHash: PhysicalMaterialPartHash): this;
    setMaterial(materialName: keyof typeof PhysicalMaterialPartHash): this;
    /**
     * Returns a `BinaryString` representation of the entity.
     *
     * @since v3.0.0
     *
     * @example
     * import { ComponentHash, Entity } from 'att-string-transcoder';
     *
     * const torchFireEntity = new Entity<'Torch'>('Fire_30100');
     *
     * const componentVersions = new Map<number, number>([
     *   [ComponentHash.NetworkRigidbody, 1],
     *   [ComponentHash.PhysicalMaterialPart, 1],
     *   [ComponentHash.Pickup, 2]
     *   // etc...
     * ]);
     *
     * const binaryString = torchFireEntity.toBinary(componentVersions);
     */
    toBinary(componentVersions: Map<number, number>): BinaryString;
    /**
     * Writes a `BinaryString` representation of the entity to the given `BinaryWriter`, including the
     * entity hash and data length.
     *
     * @since v3.0.0
     *
     * @example
     * import { BinaryWriter, ComponentHash, Entity } from 'att-string-transcoder';
     *
     * const torchFireEntity = new Entity<'Torch'>('Fire_30100');
     *
     * const componentVersions = new Map<number, number>([
     *   [ComponentHash.NetworkRigidbody, 1],
     *   [ComponentHash.PhysicalMaterialPart, 1],
     *   [ComponentHash.Pickup, 2]
     *   // etc...
     * ]);
     *
     * const writer = new BinaryWriter();
     *
     * torchFireEntity.write(writer, componentVersions);
     */
    write(writer: BinaryWriter, componentVersions: Map<number, number>): void;
}
export {};
