import type { BinaryReader } from '../BinaryReader.js';
import type { BinaryString } from '../types/BinaryString.js';
import { BinaryWriter } from '../BinaryWriter.js';
type BaseComponentProps = {
    hash: number;
    name: string;
};
export type ComponentProps = {
    version: number;
};
/**
 * Base class for all components. This class should not be used directly. Use any of its derived
 * classes instead.
 *
 * @see [Class: `Component`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/Component.md)
 * @see [API Reference Documentation](https://github.com/mdingena/att-string-transcoder/tree/main/docs/README.md)
 */
export declare class Component {
    /**
     * The hash of the component.
     *
     * @example
     * import { DerivedComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new DerivedComponent({ version: componentVersion });
     *
     * const hash = component.hash;
     */
    hash: number;
    /**
     * The name of the component.
     *
     * @example
     * import { DerivedComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new DerivedComponent({ version: componentVersion });
     *
     * const name = component.name;
     */
    name: string;
    /**
     * The version of the component. This controls the shape of the component data and its
     * (de)serialisation sequence when reading or writing binary data.
     *
     * @example
     * import { DerivedComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new DerivedComponent({ version: componentVersion });
     *
     * const version = component.version;
     * // `version` is `1`
     */
    version: number;
    /**
     * Creates a versioned component. This constructor is used internally. You shouldn't create any
     * instances of this base class.
     *
     * @see [Class: `Component`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/Component.md)
     * @see [API Reference Documentation](https://github.com/mdingena/att-string-transcoder/tree/main/docs/README.md)
     */
    constructor({ hash, name, version }: ComponentProps & BaseComponentProps);
    /**
     * Reads the binary string data and returns an instantiated component.
     *
     * @example
     * import { BinaryReader, DerivedComponent } from 'att-string-transcoder';
     *
     * const reader = new BinaryReader('...');
     * const componentVersion = 1;
     *
     * const component = DerivedComponent.fromBinary(reader, componentVersion);
     */
    static fromBinary(reader: BinaryReader, version: number, hash: number, name: string, dataLength: number): Component;
    /**
     * Returns a `BinaryString` representation of the component.
     *
     * @example
     * import { DerivedComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new DerivedComponent({ version: componentVersion });
     *
     * const binaryString = component.toBinary(componentVersion);
     */
    toBinary(_version?: number): BinaryString;
    /**
     * Writes a `BinaryString` representation of the component to the given `BinaryWriter`, including
     * the component hash and data length.
     *
     * @example
     * import { BinaryWriter, DerivedComponent } from 'att-string-transcoder';
     *
     * const writer = new BinaryWriter();
     * const componentVersion = 1;
     *
     * const component = new DerivedComponent({ version: componentVersion });
     *
     * component.write(writer, componentVersion);
     */
    write(writer: BinaryWriter, version?: number): void;
}
export {};
