import type { BinaryReader } from '../BinaryReader.js';
import type { BinaryString } from '../types/BinaryString.js';
import { Component, type ComponentProps } from './Component.js';
type Color = {
    r: number;
    g: number;
    b: number;
    a: number;
};
type Effect = null | {
    hash: number;
    strengthMultiplier: number;
};
type FoodChunk = number;
type CustomData = null | {
    color: Color;
    isConsumableThroughSkin: boolean;
    visualDataHash: number;
    effects: Effect[];
    foodChunks: FoodChunk[];
};
type LiquidContainerComponentPropsV1 = {
    canAddTo?: boolean | undefined;
    canRemoveFrom?: boolean | undefined;
    contentLevel?: number | undefined;
    hasContent?: boolean | undefined;
    isCustom?: boolean | undefined;
    presetHash?: number | undefined;
    customData?: CustomData | undefined;
};
export type LiquidContainerComponentProps = LiquidContainerComponentPropsV1;
/**
 * @see [Class: `LiquidContainerComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/LiquidContainerComponent.md)
 * @since v3.0.0
 * @version 1
 *
 * @example
 * import { LiquidContainerComponent } from 'att-string-transcoder';
 *
 * const componentVersion = 1;
 * const component = new LiquidContainerComponent({ version: componentVersion });
 */
export declare class LiquidContainerComponent extends Component {
    /**
     * Whether any content can be added to the liquid container or not.
     *
     * @since v1
     *
     * @example
     * import { LiquidContainerComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new LiquidContainerComponent({ version: componentVersion });
     *
     * const canAddTo = component.canAddTo;
     * // `canAddTo` is `false`
     */
    canAddTo: boolean;
    /**
     * Whether any content can be removed from the liquid container or not.
     *
     * @since v1
     *
     * @example
     * import { LiquidContainerComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new LiquidContainerComponent({ version: componentVersion });
     *
     * const canRemoveFrom = component.canRemoveFrom;
     * // `canRemoveFrom` is `false`
     */
    canRemoveFrom: boolean;
    /**
     * The amount of content stored inside the liquid container.
     *
     * @since v1
     *
     * @example
     * import { LiquidContainerComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new LiquidContainerComponent({ version: componentVersion });
     *
     * const contentLevel = component.contentLevel;
     * // `contentLevel` is `0`
     */
    contentLevel: number;
    /**
     * Whether any content is stored in the liquid container or not.
     *
     * @since v1
     *
     * @example
     * import { LiquidContainerComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new LiquidContainerComponent({ version: componentVersion });
     *
     * const hasContent = component.hasContent;
     * // `hasContent` is `false`
     */
    hasContent: boolean;
    /**
     * Whether the content stored in the liquid container is custom or a preset.
     *
     * @since v1
     *
     * @example
     * import { LiquidContainerComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new LiquidContainerComponent({ version: componentVersion });
     *
     * const isCustom = component.isCustom;
     * // `isCustom` is `false`
     */
    isCustom: boolean;
    /**
     * The hash of the preset of the contents stored in the liquid container. Is `0` when no preset is used (uses `CustomData` instead).
     *
     * @since v1
     *
     * @example
     * import { LiquidContainerComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new LiquidContainerComponent({ version: componentVersion });
     *
     * const presetHash = component.presetHash;
     * // `presetHash` is `0`
     */
    presetHash: number;
    /**
     * The data to use as the customised contents of the liquid container.
     *
     * @since v1
     *
     * @example
     * import { LiquidContainerComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new LiquidContainerComponent({ version: componentVersion });
     *
     * const customData = component.customData;
     * // `customData` is `null`
     */
    customData: null | CustomData;
    /**
     * @see [Class: `LiquidContainerComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/LiquidContainerComponent.md)
     * @since v3.0.0
     * @version 1
     *
     * @example
     * import { LiquidContainerComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 1;
     * const component = new LiquidContainerComponent({ version: componentVersion });
     */
    constructor({ version, canAddTo, canRemoveFrom, contentLevel, customData, hasContent, isCustom, presetHash }: ComponentProps & LiquidContainerComponentProps);
    static fromBinary(reader: BinaryReader, version: number): LiquidContainerComponent;
    toBinary(version?: number): BinaryString;
}
export {};
