import type { BinaryReader } from '../BinaryReader.js';
import type { BinaryString } from '../types/BinaryString.js';
import { LogicOperator } from '../types/LogicOperator.js';
import { Component, type ComponentProps } from './Component.js';
type LogicGateReceiverComponentPropsV2 = {
    senders?: number[] | undefined;
    operationType?: LogicOperator | undefined;
    isInversedOutputSaved?: boolean | undefined;
};
export type LogicGateReceiverComponentProps = LogicGateReceiverComponentPropsV2;
/**
 * @see [Class: `LogicGateReceiverComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/LogicGateReceiverComponent.md)
 * @since v3.0.0
 * @version 2
 *
 * @example
 * import { LogicGateReceiverComponent } from 'att-string-transcoder';
 *
 * const componentVersion = 2;
 * const component = new LogicGateReceiverComponent({ version: componentVersion });
 */
export declare class LogicGateReceiverComponent extends Component {
    /**
     * The identifiers of the `Logic*Sender` components that are connected to this `LogicGateReceiver`.
     *
     * @since v2
     *
     * @example
     * import { LogicGateReceiverComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new LogicGateReceiverComponent({ version: componentVersion });
     *
     * const senders = component.senders;
     * // `senders` is `[]`
     */
    senders: number[];
    /**
     * The type of operation to perform on the received inputs.
     *
     * @since v2
     *
     * @example
     * import { LogicGateReceiverComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new LogicGateReceiverComponent({ version: componentVersion });
     *
     * const operationType = component.operationType;
     * // `operationType` is `0` (AND)
     */
    operationType: LogicOperator;
    /**
     * Whether to inverse the output of this gate.
     *
     * @since v2
     *
     * @example
     * import { LogicGateReceiverComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new LogicGateReceiverComponent({ version: componentVersion });
     *
     * const isInversedOutputSaved = component.isInversedOutputSaved;
     * // `isInversedOutputSaved` is `false`
     */
    isInversedOutputSaved: boolean;
    /**
     * @see [Class: `LogicGateReceiverComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/LogicGateReceiverComponent.md)
     * @since v3.0.0
     * @version 2
     *
     * @example
     * import { LogicGateReceiverComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new LogicGateReceiverComponent({ version: componentVersion });
     */
    constructor({ version, senders, operationType, isInversedOutputSaved }: ComponentProps & LogicGateReceiverComponentProps);
    static fromBinary(reader: BinaryReader, version: number): LogicGateReceiverComponent;
    toBinary(version?: number): BinaryString;
}
export {};
