import type { BinaryReader } from '../BinaryReader.js';
import type { BinaryString } from '../types/BinaryString.js';
import type { Position } from '../types/Position.js';
import type { Rotation } from '../types/Rotation.js';
import { Component, type ComponentProps } from './Component.js';
type GrabPointParameters = null | {
    grabPointIndex: number;
    lastLinearPosition: number;
    lastPosition: Position;
    lastRotation: Rotation;
};
type PickupComponentPropsV1 = {
    lastInteractorPlayerId?: number | undefined;
    dockedMemory?: never;
};
type PickupComponentPropsV2 = {
    lastInteractorPlayerId?: number | undefined;
    dockedMemory?: GrabPointParameters[] | undefined;
};
export type PickupComponentProps = PickupComponentPropsV1 | PickupComponentPropsV2;
/**
 * @see [Class: `BasicDecayComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/BasicDecayComponent.md)
 * @since v3.0.0
 * @version 2
 *
 * @example
 * import { PickupComponent } from 'att-string-transcoder';
 *
 * const componentVersion = 2;
 * const component = new PickupComponent({ version: componentVersion });
 */
export declare class PickupComponent extends Component {
    /**
     * The ID of the player that last interacted with the prefab that this component is attached to.
     *
     * @since v1
     *
     * @example
     * import { PickupComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new PickupComponent({ version: componentVersion });
     *
     * const lastInteractorPlayerId = component.lastInteractorPlayerId;
     * // `lastInteractorPlayerId` is `0`
     */
    lastInteractorPlayerId: number;
    /**
     * An array of `<GrabPointParameters>` that describes the orientation of prefabs when they were
     * docked. Used to control in which orientation pickups should be held when grabbed from the dock
     * attached to the same prefab this component is attached to.
     *
     * @since v2
     *
     * @example
     * import { PickupComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new PickupComponent({ version: componentVersion });
     *
     * const dockedMemory = component.dockedMemory;
     * // `dockedMemory` is `[]`
     */
    dockedMemory: GrabPointParameters[];
    /**
     * @see [Class: `BasicDecayComponent`](https://github.com/mdingena/att-string-transcoder/tree/main/docs/BasicDecayComponent.md)
     * @since v3.0.0
     * @version 2
     *
     * @example
     * import { PickupComponent } from 'att-string-transcoder';
     *
     * const componentVersion = 2;
     * const component = new PickupComponent({ version: componentVersion });
     */
    constructor({ version, lastInteractorPlayerId, dockedMemory }: ComponentProps & PickupComponentProps);
    static fromBinary(reader: BinaryReader, version: number): PickupComponent;
    toBinary(version?: number): BinaryString;
}
export {};
